import greenfoot.*;

// Piece class: creates object repesenting chess pieces; allows user to drag'n'drop the objects to make valid moves
public class Piece extends Actor
{
    private int pceType = 0; // its value represents the type of piece 'this' is (-6 to 6)
    private boolean isSelected = false; // set true when dragging
    private String fromSq = ""; // String representation of the square the piece started on ("A1" format)
    private String toSquares = ""; // String representation of the squares the piece can end on ("A1" format)
    // toSquares (above) is a concatenated list of squares in "A1" format, without any seperation characters
    
    // Piece(String) - constructor for Piece.class - takes a single-character String representing the type of piece to create
    // and saves the type of piece in number form (-6 to 6) and sets the image for the object
    public Piece(String pceCode)
    {
        pceType = "kqrbnp PNBRQK".indexOf(pceCode.charAt(0)) - 6; // gets an integer representation of piece type
        GreenfootImage pceImg = new GreenfootImage("" + (char) Board.pieceInt[pceType + 6], 72, Color.BLACK, new Color(0, 0, 0, 0)); // creates piece image
        GreenfootImage image = new GreenfootImage(75, 75); // creates the base image (to become the main image)
        image.drawImage(pceImg, (image.getWidth() - pceImg.getWidth()) / 2, (image.getHeight() - pceImg.getHeight()) / 2); // adds the piece image to the main image
        setImage(image); // sets the image of the object to that of the main image
    }

    // act() controls dragging and selecting of moves for 'this' piece
    public void act() 
    {
        Board board = (Board) getWorld(); // gets a reference to the world
        if (isTurn())
        { // piece belongs to that of the player who currently has the move
            if (!isSelected && Greenfoot.mousePressed(this)) // new piece selection
            { // piece is currently unselected and mouse button pressed while over this piece
                isSelected = true; // flags the piece as selected
                fromSq = notateSquare(getX(), getY()); // saves the square the piece currently occupies
                String allMvs = board.moveList + ":"; // gets a copy of all moves currently available to make from the world
                toSquares = ""; // clears the String to build a new list of squares this piece can currently move
                while (allMvs.indexOf(":" + fromSq) != -1)
                { // while moves remain in the list that start from the square this piece occupies
                    allMvs = allMvs.substring(allMvs.indexOf(":" + fromSq) + 1); // removes all moves before the first matching move found
                    toSquares = toSquares + allMvs.substring(2, 4); // adds the destination square to the list of squares this piece can move
                }
                if (toSquares.length() == 0) isSelected = false; // if no moves listed for this piece, flag the piece as unselected
                return; // exits current execution of this 'act' method
            }
            if (isSelected && Greenfoot.mouseDragged(this)) // follow the mouse
            { // piece is currently selected and the mouse is dragging this piece
                MouseInfo mi = Greenfoot.getMouseInfo(); // gets mouse information
                setLocation(mi.getX(), mi.getY()); // sets location of piece at location of mouse
                return; // exits current execution of this 'act' method
            }
            if (isSelected && Greenfoot.mouseDragEnded(this)) // select the move
            { // piece is currently selected and the mouse button was released after dragging this piece
                isSelected = false; // flags the piece as unselected
                MouseInfo mi = Greenfoot.getMouseInfo(); // gets mouse infomation
                String toSq = notateSquare(mi.getX(), mi.getY()); // creates String representation of square mouse drag ended at
                if (toSquares.indexOf(toSq) == -1) // final square not in list of 'toSquares'
                { // square mouse ended at is not in list of valid squares the piece can currently move to
                    setLocation(realX(fromSq), realY(fromSq)); // returns piece to location from which it started
                    return; // exits current execution of this 'act' method
                }
                board.makeMove(fromSq, toSq); // move valid, execute move
                return; // exits current execution of this 'act' method
            }
            if (isSelected && Greenfoot.mouseClicked(this)) // re-adjusts position of slightly dragged pieces
            { // piece is currently selected and mouse button was released without dragging being registered
                isSelected = false; // flags piece as unselected
                setLocation(realX(fromSq), realY(fromSq)); // centers the piece on its square
            }
        }
    }

    // notateSquare(int, int) takes the real world coordinates,
    // converts them to a String represention of the square those coordinates fall upon
    // and returns that String ("A1" format)
    private String notateSquare(int xVal, int yVal)
    {
        int myX = (xVal - (xVal % 75)) / 75; // calculates x-coordinate on board
        if (myX < 0) myX = 0; else if (myX > 7) myX = 7; // selects nearest x-location if outside board boundaries
        int myY = 7 - ((yVal - (yVal % 75)) / 75); // calculates y-coodinate on board
        if (myY < 0) myY = 0; else if (myY > 7) myY = 7; // selects nearest y-location if outside board boundaries
        return ("abcdefgh".substring(myX, myX + 1) + "12345678".substring(myY, myY + 1)); // returns String representing the square (or closest square)
    }

    // isTurn() returns true if 'this' piece belongs to the player who has the move, else returns false
    private boolean isTurn() { return ((1 - 2 * (((Board) getWorld()).moveCount % 2)) * pceType > 0); }
    
    // realX(String) takes a String representation of a square ("A1" format),
    // converts the column (0 to 7) number, or rank number, of that square to a real world coordinate
    // and returns the coordinate of the point at the center of that square
    private int realX(String sq) { return getFile(sq) * 75 + 37; }
    
    // realY(String) takes a String representation of a square ("A1" format),
    // converts the row (0 to 7) number, or file number, of that square to a real world coordinate
    // and returns the coordinate of the point at the center of the square
    private int realY(String sq) { return getRank(sq) * 75 + 37; }
    
    // getFile(String) takes a String representation of a square ("A1" format)
    // and returns the row (0 to 7) number, or file number, of that square
    private int getFile(String sq) { return "abcdefgh".indexOf(sq.charAt(0)); }
    
    // getRank(String) takes a String representation of a square ("A1" format)
    // and returns the column (0 to 7) number, or rank number, of that square
    private int getRank(String sq) { return 7 - "12345678".indexOf(sq.charAt(1)); }
}